include "Port.inc"
include "ZeroPage.inc"
include "Vram.inc"
include "VVram.inc"
include "Chars.inc"

ext BackgroundPattern
ext VVram_

PatternSize equ VramStep*CharHeight
WindowTop equ Vram+VramRowSize*0

dseg
Backup: public Backup
    defs WindowWidth*VVramHeight

pVVram equ ZW0
pVram equ ZW1
pBackup equ ZW2
pPattern equ ZW3

yCount equ ZB0


cseg
InitVram: public InitVram
    lda #160 | sta LCD_X_Size
    lda #160 | sta LCD_Y_Size
    stz X_Scroll
    stz Y_Scroll
rts


; void ClearScreen();
cseg
ClearScreen_: public ClearScreen_
    pha
        lda #low VramOrg | sta pVram
        lda #high VramOrg | sta pVram+1
        ldx #$1e
        lda #0
        do
            ldy #0
            do
                sta (<pVram),y
                iny
            while ne | wend
            inc <pVram+1
            dex
        while ne | wend

        lda #low Backup | sta <pBackup
        lda #high Backup | sta <pBackup+1
        ldx #VVramHeight
        do
            ldy #0 | tya
            do
                sta (<pBackup),y
                iny
                cpy #WindowWidth
            while ne | wend
            lda <pBackup | clc|adc #WindowWidth | sta <pBackup
            lda <pBackup+1 | adc #0 | sta <pBackup+1
            dex
        while ne | wend
    pla
rts


cseg
PatternPtr:
    tax
        stz <pPattern+1
        asl a | rol <pPattern+1 ;*2
        asl a | rol <pPattern+1 ;*4
        asl a | rol <pPattern+1 ;*8
        asl a | rol <pPattern+1 ;*16
        sta <pPattern
    txa
    lda <pPattern | clc|adc #low BackgroundPattern | sta <pPattern
    lda <pPattern+1 | adc #high BackgroundPattern | sta <pPattern+1
rts
Put: ;(,a)
    jsr PatternPtr    
    ldx #CharHeight
    do
        ldy #0
        do
            lda (<pPattern),y
            sta (<pVram),y
            iny
            cpy #VramStep
        while ne | wend
        lda <pPattern | clc|adc #VramStep | sta <pPattern
        lda <pPattern+1 | adc #0 | sta <pPattern+1

        lda <pVram | clc|adc #VramWidth | sta <pVram
        lda <pVram+1 | adc #0 | sta <pVram+1

        dex
    while ne | wend
    lda <pVram | clc|adc #low(VramStep-VramWidth*CharHeight) | sta <pVram
    lda <pVram+1 | adc #high(VramStep-VramWidth*CharHeight) | sta <pVram+1
rts


; ptr<byte> Put(ptr<byte> pVram, byte c);
dseg
Put_@Param0: public Put_@Param0
Put_pVram:
PrintC_@Param0: public PrintC_@Param0
PrintC_pVram:
    defw 0
Put_@Param1: public Put_@Param1
Put_c:
PrintC_@Param1: public PrintC_@Param1
PrintC_c:
    defb 0
cseg
Put_: public Put_
    pha
        lda Put_pVram | sta <pVram
        lda Put_pVram+1 | sta <pVram+1
        lda Put_c
        jsr Put

        lda Put_pVram | clc|adc #VramStep | tay
        lda Put_pVram+1 | adc #0 | tax
    pla
rts


; ptr<byte> PrintC(ptr<byte> pVram, byte c);
cseg
CharTable:
    defb " -0123456789ABCEFGHIMNOPRSTUV"
PrintC_: public PrintC_
    pha
        lda PrintC_pVram | sta <pVram
        lda PrintC_pVram+1 | sta <pVram+1
        
        lda PrintC_c
        ldx #0
        do
            cmp CharTable,x
            beq PrintC_break
            inx
            cpx #29
        while ne | wend
        PrintC_break:
        txa
        jsr Put

        lda PrintC_pVram | clc|adc #VramStep | tay
        lda PrintC_pVram+1 | adc #0 | tax
    pla
rts


; void VVramToVram();
cseg
VVramToVram_: public VVramToVram_
    pha
        lda #low WindowTop | sta <pVram
        lda #high WindowTop | sta <pVram+1

        lda #low VVram_  | sta <pVVram
        lda #high VVram_ | sta <pVVram+1
        
        lda #low Backup | sta <pBackup
        lda #high Backup | sta <pBackup+1

        lda #VVramHeight | sta <yCount
        do
            ldy #0
            do
                lda (<pVVram),y
                cmp (<pBackup),y
                if ne
                    sta (<pBackup),y
                    phy
                        jsr Put
                    ply
                else
                    lda <pVram | clc|adc #VramStep | sta <pVram
                    lda <pVram+1 | adc #0 | sta <pVram+1
                endif
                iny
                cpy #WindowWidth
            while ne | wend
            lda <pVram | clc|adc #low(VramRowSize-VramStep*WindowWidth) | sta <pVram
            lda <pVram+1 | adc #high(VramRowSize-VramStep*WindowWidth) | sta <pVram+1

            lda <pVVram | clc|adc #VVramWidth | sta <pVVram
            lda <pVVram+1 | adc #0 | sta <pVVram+1

            lda <pBackup | clc|adc #WindowWidth | sta <pBackup
            lda <pBackup+1 | adc #0 | sta <pBackup+1

            dec <yCount
        while ne | wend
    pla
rts
